##########################################################
# grid functions
##########################################################    

from numarray import *
import mapping
import libutil
import treelib
import myNumeric

'''

definition of indexes and corresponding physical values

ix   =  int( (x-xmin)/(xmax-xmin)* nx )
x    = ix*(xmax-xmin)/(nx) + xmin 

center of cell

xc    = (ix+0.5)*(xmax-xmin)/(nx) + xmin 

physical size of cells

dx    = ((arange(nx)+1)*(xmax-xmin)/(nx) + xmin ) 
      - ((arange(nx)  )*(xmax-xmin)/(nx) + xmin ) 
      
      = (xmax-xmin)/nx * ones(nx)
      

# linear density of spherical density

lambda(r) = 4*pi*r**2*rho(r)




here we may deal the following grids :

  - carthesian 2d
  - carthesian 3d

  - cylindrical 2d		
  - cylindrical 3d
  
  - spherical	3d




!!! when computing the potential, in some 1d/2d grid,
!!! we do not take into account the fact that the model
!!! may be assymetric
!!! We should better compute potential on 3d grid
!!! and take the mean.

!!! the cylindrical grid is not symetric in z

'''


#######################################
# general functions
#######################################

def get_First_Derivative(f,x):
  '''
  First derivative of f(x)
  '''
  
  fp = zeros(len(x),x.type())
    
  fp[0 ]  = (f[ 1]-f[ 0])/(x[ 1]-x[ 0]) 
  fp[-1]  = (f[-1]-f[-2])/(x[-1]-x[-2])
  
  
  f1 = f[2:]
  f2 = f[:-2]
  x1 = x[2:]
  x2 = x[:-2]  
  
  fp[1:-1] = (f1-f2)/(x1-x2)
  
  return fp


def get_Mean_Along_Axis(mat,axis=0):

  if len(mat.shape) == 1:
    
    return mat

  elif len(mat.shape) == 2:
    
    a = fmod((axis+1),2)
    n = mat.shape[a]
    
    s = mat
    s = sum(s,axis=a)/n
    
  elif len(mat.shape) == 3:
    
    a = array([fmod((axis+1),3),fmod((axis+2),3)])
    a.sort()
    a1 = a[0]
    a2 = a[1]     
    n1 = mat.shape[a1]
    n2 = mat.shape[a2]
    
    s = mat
    s = sum(s,axis=a2)/n2
    s = sum(s,axis=a1)/n1

  return s


def get_Sum_Along_Axis(mat,axis=0):

  if len(mat.shape) == 1:
    
    return mat
    
  elif len(mat.shape) == 2:
    
    a = fmod((axis+1),2)
    
    s = mat
    s = sum(s,axis=a)
    
  elif len(mat.shape) == 3:
    
    a = array([fmod((axis+1),3),fmod((axis+2),3)])
    a.sort()
    a1 = a[0]
    a2 = a[1]     
    
    s = mat
    s = sum(s,axis=a2)
    s = sum(s,axis=a1)

  return s


def get_Accumulation_Along_Axis(mat,axis=0):
  '''
  Accumulate values along an axis
  '''
  v = get_Sum_Along_Axis(mat,axis=axis) 
  return add.accumulate(v)
  


def get_Integral(v,dr,ia,ib):
  '''
  Integrate the vector v, between ia and ib.
  
  v  : values of cells (must be 1 dimensional)
  dr : corresponding physical size of cells
  ia : lower  real indice
  ib : higher real indice
  '''
  
  ia = max(0,ia)
  ib = min(len(v),ib)
      
  
  if ia==ib:
    return 0.0
  
  if ia>ib:
    raise "ia must be < ib"  
  
  
  iap = int(ceil(ia))
  ibp = int(floor(ib))
  
  dra = iap-ia
  drb = ib-ibp  
    
  Ia = 0.0
  if dra != 0:
    Ia = v[iap-1] * dra
  
  Ib = 0.0  
  if drb != 0:
    Ib = v[ibp] * drb 
  
  
  I = v[iap:ibp]*dr[iap:ibp]

        
  return sum(I)+Ia+Ib
  
  
def get_Symetrisation_Along_Axis(mat,axis=1):
  '''
  Return an array where the two half are symetrized
  '''
  
  nx,ny = mat.shape
  
  odd = fmod(ny,2)
  
  if odd:
    mat1 = mat[:,0:(ny-1)/2]
    mat2 = mat[:,(ny-1)/2+1:] 
    mat3 = mat[:,(ny-1)/2] 
  else:
    mat1 = mat[:,0:ny/2]
    mat2 = mat[:,ny/2:]
  
  mat2 = myNumeric.turnup(mat2,0)

  # take the mean
  matm = (mat1+mat2) /2

  mat = ones(mat.shape,mat.type())
  
  if odd:
    
    mat[:,0:(ny-1)/2]  = matm
    mat[:,(ny-1)/2+1:] = myNumeric.turnup(matm,0)
    mat[:,(ny-1)/2]    = mat3
  else:
    mat[:,0:ny/2]      = matm
    mat[:,ny/2:]       = myNumeric.turnup(matm,0)
    
  return mat






  




#######################################
# carthesian grid
#######################################

##############
# 2 dimensions
##############

def get_xy_Of_Carthesian_2d_Grid(nx,ny,xmin,xmax,ymin,ymax,offx=0,offy=0):

  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
    
  ix = arange(nx) 
  iy = arange(ny)
 
  x = (ix+offx)*(xmax-xmin)/nx + xmin  
  y = (iy+offy)*(ymax-ymin)/ny + ymin 

  return x,y
  

def get_Points_On_Carthesian_2d_Grid(nx,ny,xmin,xmax,ymin,ymax,offx=0,offy=0):
  '''
  Return an array of points corresponding to the center of cells
  af a 2d carthesian grid.
  
  To get a nt X nr array from the returned vector (pos), do
  
  x = copy.copy(pos[:,0])
  y = copy.copy(pos[:,1])
  z = copy.copy(pos[:,2])

  x.shape = (nx,ny)
  y.shape = (nx,ny)
  z.shape = (nx,ny)
    
  '''

  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
  nx = int(nx)
  ny = int(ny)  
      
  ix,iy = indices((nx,ny))

  x = (ix+offx)*(xmax-xmin)/nx + xmin  
  y = (iy+offy)*(ymax-ymin)/ny + ymin  

  x = ravel(x)
  y = ravel(y)
  z = zeros(len(x))

  return transpose(array([x,y,z])).astype(Float32)
  

def get_MassMap_On_Carthesian_2d_Grid(nb,nx,ny,xmin,xmax,ymin,ymax):
  '''
  Return an array of points containing mass of particles
  '''
  
  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
  
  x = nb.pos[:,0]
  y = nb.pos[:,1]
  z = nb.pos[:,2]
  
  # scale between 0 and 1
  x   = (x-xmin)/(xmax-xmin)
  y   = (y-ymin)/(ymax-ymin)
  
  # compute values
  pos = transpose(array([x,y,z])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= nb.mass.astype(Float32) 
  shape = (nx,ny)
  
  # make the map  
  mat = mapping.mkmap2d(pos,mass,val,shape)
  
  return mat

def get_NumberMap_On_Carthesian_2d_Grid(nb,nx,ny,xmin,xmax,ymin,ymax):
  '''
  Return an array of points containing mass of particles
  '''
  
  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
  
  x = nb.pos[:,0]
  y = nb.pos[:,1]
  z = nb.pos[:,2]
  
  # scale between 0 and 1
  x   = (x-xmin)/(xmax-xmin)
  y   = (y-ymin)/(ymax-ymin)
  
  # compute values
  pos = transpose(array([x,y,z])).astype(Float32)
  val = ones(pos.shape).astype(Float32)
  mass= ones(pos.shape).astype(Float32)
  shape = (nx,ny)
  
  # make the map  
  mat = mapping.mkmap2d(pos,mass,val,shape)
  
  return mat

def get_PotentialMap_On_Carthesian_2d_Grid(nb,nx,ny,xmin,xmax,ymin,ymax,eps,Tree=None):
  '''
  Return an array of points containing potential   
  '''

  pos = get_Points_On_Carthesian_2d_Grid(nx,ny,xmin,xmax,ymin,ymax)
  
  Tree = nb.getTree()
    
  pot = Tree.Potential(pos,eps)
  
  # transform back into array  
  pot.shape = (nx,ny)

  return pot  
  
def get_SurfaceMap_On_Carthesian_2d_Grid(nb,nx,ny,xmin,xmax,ymin,ymax):
  '''
  Return an array of points containing corresponding physical 
  volumes of each cell (usefull to compute density)
    
  '''
  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
  
  # compute volume (surface)
  dx = (xmax-xmin)/nx 
  dy = (ymax-ymin)/ny 
  v  = dx*dy 
  
  # make the map  
  mat = ones((nx,ny)) * v
  mat = mat.astype(Float32)
  
  return mat


def get_SurfaceDensityMap_On_Carthesian_2d_Grid(nb,nx,ny,xmin,xmax,ymin,ymax):
  '''
  Return an array of points containing density in each cell
  '''
    
  m = get_MassMap_On_Carthesian_2d_Grid(nb,nx,ny,xmin,xmax,ymin,ymax)
  s = get_SurfaceMap_On_Carthesian_2d_Grid(nb,nx,ny,xmin,xmax,ymin,ymax)
  
  return m/s

##############
# 3 dimensions
##############
  
def get_xyz_Of_Carthesian_3d_Grid(nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax,offx=0,offy=0,offz=0):

  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
  zmax = float(zmax)
  zmin = float(zmin)
      
  ix = arange(nx)
  iy = arange(ny)
  iz = arange(nz)
 
  x = (ix+offx)*(xmax-xmin)/nx + xmin  
  y = (iy+offy)*(ymax-ymin)/ny + ymin 
  z = (iz+offz)*(zmax-zmin)/nz + zmin 

  return x,y,z


def get_Points_On_Carthesian_3d_Grid(nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax,offx=0,offy=0,offz=0):
  '''
  Return an array of points corresponding to the center of cells
  af a 3d carthesian grid.
  
  To get a nt X nr array from the returned vector (pos), do
  
  x = copy.copy(pos[:,0])
  y = copy.copy(pos[:,1])
  z = copy.copy(pos[:,2])

  x.shape = (nx,ny,nz)
  y.shape = (nx,ny,nz)
  z.shape = (nx,ny,nz)
    
  '''

  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
  zmax = float(zmax)
  zmin = float(zmin)
  nx = int(nx)
  ny = int(ny)    
  nz = int(nz)
      
  ix,iy,iz = indices((nx,ny,nz))

  x = (ix+offx)*(xmax-xmin)/nx + xmin  
  y = (iy+offy)*(ymax-ymin)/ny + ymin  
  z = (iz+offz)*(zmax-zmin)/nz + zmin

  x = ravel(x)
  y = ravel(y)
  z = ravel(z)

  return transpose(array([x,y,z])).astype(Float32)


def get_MassMap_On_Carthesian_3d_Grid(nb,nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax):
  '''
  Return an array of points containing mass of particles
  '''
  
  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
  zmax = float(zmax)
  zmin = float(zmin)
    
  x = nb.pos[:,0]
  y = nb.pos[:,1]
  z = nb.pos[:,2]
  
  # scale between 0 and 1
  x   = (x-xmin)/(xmax-xmin)
  y   = (y-ymin)/(ymax-ymin)
  z   = (z-zmin)/(zmax-zmin)
  
  # compute values
  pos = transpose(array([x,y,z])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= nb.mass.astype(Float32) 
  shape = (nx,ny,nz)
  
  # make the map  
  mat = mapping.mkmap3d(pos,mass,val,shape)
  
  return mat

def get_NumberMap_On_Carthesian_3d_Grid(nb,nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax):
  '''
  Return an array of points containing mass of particles
  '''
  
  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
  zmax = float(zmax)
  zmin = float(zmin)
    
  x = nb.pos[:,0]
  y = nb.pos[:,1]
  z = nb.pos[:,2]
  
  # scale between 0 and 1
  x   = (x-xmin)/(xmax-xmin)
  y   = (y-ymin)/(ymax-ymin)
  z   = (z-zmin)/(zmax-zmin)
  
  # compute values
  pos = transpose(array([x,y,z])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= ones(pos.shape).astype(Float32)
  shape = (nx,ny,nz)
  
  # make the map  
  mat = mapping.mkmap3d(pos,mass,val,shape)
  
  return mat

def get_PotentialMap_On_Carthesian_3d_Grid(nb,nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax,eps,Tree=None):
  '''
  Return an array of points containing potential   
  '''

  pos = get_Points_On_Carthesian_3d_Grid(nb,nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax)
  
  Tree = nb.getTree()
    
  pot = Tree.Potential(pos,eps)
  
  # transform back into array  
  pot.shape = (nx,ny,nz)

  
  return pot  

def get_VolumeMap_On_Carthesian_3d_Grid(nb,nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax):
  '''
  Return an array of points containing corresponding physical 
  volumes of each cell (usefull to compute density)
  '''
  xmax = float(xmax)
  xmin = float(xmin)
  ymax = float(ymax)
  ymin = float(ymin)
  zmax = float(zmax)
  zmin = float(zmin)
    
  # compute volume (surface)
  dx = (xmax-xmin)/nx 
  dy = (ymax-ymin)/ny 
  dz = (zmax-zmin)/nz 
  v  = dx*dy*dz 
  
  # make the map  
  mat = ones((nx,ny,nz)) * v
  mat = mat.astype(Float32)
  
  return mat


def get_DensityMap_On_Carthesian_3d_Grid(nb,nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax):
  '''
  Return an array of points containing density in each cell
  '''
    
  m = get_MassMap_On_Carthesian_3d_Grid(nb,nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax)
  v = get_VolumeMap_On_Carthesian_3d_Grid(nb,nx,ny,nz,xmin,xmax,ymin,ymax,zmin,zmax)
  
  return m/v

#######################################
# cylindrical grid
#######################################


######################
# 2 dimensions r,t (h)
######################

def get_rt_Of_Cylindrical_2dh_Grid(nr,nt,rmax,offr=0,offt=0):

  rmin = 0.0
  rmax = float(rmax)
  tmin = 0.0
  tmax = 2*pi
      
  ir = arange(nr) 
  it = arange(nt)
 
  r = (ir+offr)*(rmax-rmin)/nr + rmin  
  t = (it+offt)*(tmax-tmin)/nt + tmin 

  return r,t


def get_Points_On_Cylindrical_2dh_Grid(nr,nt,rmax,offr=0,offt=0):
  '''
  Return an array of points corresponding to the nodes of
  a 2d cylindrical grid
  
  To get a nt X nr array from the returned vector (pos), do
  
  x = copy.copy(pos[:,0])
  y = copy.copy(pos[:,1])
  z = copy.copy(pos[:,2])

  x.shape = (nr,nt)
  y.shape = (nr,nt)
  z.shape = (nr,nt)
  
  # to get r and theta
  r = sqrt(x**2+y**2+z**2)
  t = arctan2(y,x)*180/pi
  
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  tmin = 0.0
  tmax = 2*pi
  nr   = int(nr)
  nt   = int(nt)
    
  ir,it = indices((nr,nt))
  
  r = (ir+offr)*(rmax-rmin)/nr + rmin
  t = (it+offt)*(tmax-tmin)/nt + tmin

  r = ravel(r)
  t = ravel(t)

  x = r*cos(t)
  y = r*sin(t)
  z = zeros(len(x))

  return transpose(array([x,y,z])).astype(Float32)


def get_MassMap_On_Cylindrical_2dh_Grid(nb,nr,nt,rmax):
  '''
  Return an array of points containing mass of particles
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  tmin = 0.0
  tmax = 2*pi
  
  r = nb.rxy()
  t = nb.phi_xy() + pi
  z = nb.pos[:,2]
    
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  t   = (t-tmin)/(tmax-tmin)
  
  # compute values
  pos = transpose(array([r,t,z])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= nb.mass.astype(Float32) 
  shape = (nr,nt)
  
  # make the map  
  mat = mapping.mkmap2d(pos,mass,val,shape)
  
  return mat

def get_NumberMap_On_Cylindrical_2dh_Grid(nb,nr,nt,rmax):
  '''
  Return an array of points containing mass of particles
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  tmin = 0.0
  tmax = 2*pi
  
  r = nb.rxy()
  t = nb.phi_xy() + pi
  z = nb.pos[:,2]
    
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  t   = (t-tmin)/(tmax-tmin)
  
  # compute values
  pos = transpose(array([r,t,z])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= ones(pos.shape).astype(Float32)
  shape = (nr,nt)
  
  # make the map  
  mat = mapping.mkmap2d(pos,mass,val,shape)
  
  return mat


def get_PotentialMap_On_Cylindrical_2dh_Grid(nb,nr,nt,rmax,eps,Tree=None):
  '''
  Return an array of points containing potential   
  '''

  pos = get_Points_On_Cylindrical_2dh_Grid(nr,nt,rmax)
  
  Tree = nb.getTree()
    
  pot = Tree.Potential(pos,eps)
  
  # transform back into array  
  pot.shape = (nr,nt)
  
  return pot  


def get_SurfaceMap_On_Cylindrical_2dh_Grid(nb,nr,nt,rmax):
  '''
  Return an array of points containing corresponding physical 
  volumes of each cell (usefull to compute density)
  '''
  
  rmax = float(rmax)
    
  # compute volume (surface)
  #dr = rmax/nr
  #rs = arange(nr+1)*dr
  #v = (pi*rs[1:]**2 - pi*rs[:-1]**2 )/nt
     
  def volume(ir,it):
    dr = (rmax/nr)
    r  = ir*dr
    return pi*((r+dr)**2 - (r)**2 )/nt
 
  # make the map  
  mat = fromfunction(volume,(nr,nt))
  mat = mat.astype(Float32)
    
  return mat

def get_SurfaceDensityMap_On_Cylindrical_2dh_Grid(nb,nr,nt,rmax):
  '''
  Return an array of points containing density in each cell
  '''
    
  m = get_MassMap_On_Cylindrical_2dh_Grid(nb,nr,nt,rmax)
  s = get_SurfaceMap_On_Cylindrical_2dh_Grid(nb,nr,nt,rmax)
  
  return m/s

######################
# 2 dimensions r,z (v)
######################

def get_rz_Of_Cylindrical_2dv_Grid(nr,nz,rmax,zmin,zmax):

  rmin = 0.0
  rmax = float(rmax)
  zmin = float(zmin)
  zmax = float(zmax)
  nr   = int(nr)
  nz   = int(nz)
      
  ir = arange(nr) 
  iz = arange(nz)
 
  r = (ir+0.0)*(rmax-rmin)/nr + rmin  
  z = (iz+0.0)*(zmax-zmin)/nz + zmin 

  return r,z


def get_Points_On_Cylindrical_2dv_Grid(nr,nz,rmax,zmin,zmax,offr=0,offz=0):
  '''
  Return an array of points corresponding to the nodes of
  a 2d cylindrical grid
  
  To get a nt X nr array from the returned vector (pos), do
  
  x = copy.copy(pos[:,0])
  y = copy.copy(pos[:,1])
  z = copy.copy(pos[:,2])

  x.shape = (nr,nt)
  y.shape = (nr,nt)
  z.shape = (nr,nt)
  
  # to get r and theta
  r = sqrt(x**2+y**2+z**2)
  t = arctan2(y,x)*180/pi
  
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  zmin = float(zmin)
  zmax = float(zmax)
  nr   = int(nr)
  nz   = int(nz)
    
  ir,iz = indices((nr,nz))
  
  r = (ir+offr)*(rmax-rmin)/nr + rmin
  z = (iz+offz)*(zmax-zmin)/nz + zmin

  r = ravel(r)
  z = ravel(z)

  x = r
  y = zeros(len(x))
  z = z

  return transpose(array([x,y,z])).astype(Float32)


def get_MassMap_On_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax):
  '''
  Return an array of points containing mass of particles
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  zmin = float(zmin)
  zmax = float(zmax)
  nr   = int(nr)
  nz   = int(nz)
  
  r = nb.rxy()
  t = nb.phi_xy() + pi
  z = nb.pos[:,2]
    
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  z   = (z-zmin)/(zmax-zmin)
  
  # compute values
  pos = transpose(array([r,z,t])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= nb.mass.astype(Float32) 
  shape = (nr,nz)
  
  # make the map  
  mat = mapping.mkmap2d(pos,mass,val,shape)
  
  return mat

def get_NumberMap_On_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax):
  '''
  Return an array of points containing mass of particles
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  zmin = float(zmin)
  zmax = float(zmax)
  nr   = int(nr)
  nz   = int(nz)
  
  r = nb.rxy()
  t = nb.phi_xy() + pi
  z = nb.pos[:,2]
    
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  z   = (z-zmin)/(zmax-zmin)
  
  # compute values
  pos = transpose(array([r,z,t])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= ones(pos.shape).astype(Float32)
  shape = (nr,nz)
  
  # make the map  
  mat = mapping.mkmap2d(pos,mass,val,shape)
  
  return mat

def get_PotentialMap_On_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax,eps,Tree=None):
  '''
  Return an array of points containing potential   
  '''  
  pos = get_Points_On_Cylindrical_2dv_Grid(nr,nz,rmax,zmin,zmax)
  
  Tree = nb.getTree()
    
  pot = Tree.Potential(pos,eps)
  
  # transform back into array  
  pot.shape = (nr,nz)
  
  return pot  

def get_AccelerationMap_On_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax,eps,Tree=None):
  '''
  Return an array of points containing accelerations   
  '''  
  pos = get_Points_On_Cylindrical_2dv_Grid(nr,nz,rmax,zmin,zmax)
  
  Tree = nb.getTree()
    
  acc = Tree.Acceleration(pos,eps)
  
  accx = copy.copy(acc[:,0])
  accy = copy.copy(acc[:,1])
  accz = copy.copy(acc[:,2])
  
  # transform back into array  
  accx.shape = (nr,nz)
  accy.shape = (nr,nz)
  accz.shape = (nr,nz)
  
  return accx,accy,accz  

def get_VolumeMap_On_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax):
  '''
  Return an array of points containing corresponding physical 
  volumes of each cell (usefull to compute density)
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  zmin = float(zmin)
  zmax = float(zmax)
  nr   = int(nr)
  nz   = int(nz)
  
    
  # compute volume (surface)
  #dr = rmax/nr
  #rs = arange(nr+1)*dr
  #v = (pi*rs[1:]**2 - pi*rs[:-1]**2 )/nt
     
  def volume(ir,iz):
    dr = (rmax-rmin)/nr
    dz = (rmax-rmin)/nz
    r  = ir*dr
    return pi*((r+dr)**2 - (r)**2 ) * dz
 
  # make the map  
  mat = fromfunction(volume,(nr,nz))
  mat = mat.astype(Float32)
    
  return mat

def get_DensityMap_On_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax):
  '''
  Return an array of points containing density in each cell
  '''
    
  m = get_MassMap_On_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax)
  v = get_VolumeMap_On_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax)
  
  return m/v

def get_SurfaceDensityMap_From_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax):
  '''
  Return an array of points containing the surface density along r
  '''
    
  m = get_MassMap_On_Cylindrical_2dv_Grid(nb,nr,nz,rmax,zmin,zmax)
  m = get_Sum_Along_Axis(m)
  
  def surface(ir):
    dr = (rmax)/nr
    r  = ir*dr
    return pi*((r+dr)**2 - (r)**2 )
    
  s = fromfunction(surface,(nr,))
  
  return m/s

def get_Interpolation_On_Cylindrical_2dv_Grid(pos,mat,nr,nz,rmax,zmin,zmax,offr=0,offz=0):
  '''
  Interpolates continuous value of pos, using matrix mat
  '''
    
  rmin = 0.0
  rmax = float(rmax)
  zmin = float(zmin)
  zmax = float(zmax)
  nr   = int(nr)
  nz   = int(nz)
    
  r  = sqrt(pos[:,0]**2+pos[:,1]**2)
  z  = pos[:,2]
  
  ir = (r-rmin)/(rmax-rmin)* nr - offr
  iz = (z-zmin)/(zmax-zmin)* nz - offz
        
  return myNumeric.Interpolate_From_2d_Array(ir.astype(Float32),iz.astype(Float32),mat.astype(Float32))

def get_r_Interpolation_On_Cylindrical_2dv_Grid(pos,mat,nr,nz,rmax,zmin,zmax,offr=0):
  '''
  Interpolates continuous value of pos, using matrix mat
  only along first axis.
  '''
    
  rmin = 0.0
  rmax = float(rmax)
  zmin = float(zmin)
  zmax = float(zmax)
  nr   = int(nr)
  nz   = int(nz)
    
  r  = sqrt(pos[:,0]**2+pos[:,1]**2)
  
  ir = (r-rmin)/(rmax-rmin)* nr - offr
          
  return myNumeric.Interpolate_From_1d_Array(ir.astype(Float32),mat.astype(Float32))
  
##############
# 3 dimensions
##############

def get_rtz_Of_Cylindrical_3d_Grid(nr,nt,nz,rmax,zmin,zmax,offr=0,offt=0,offz=0):

  rmin = 0.0
  rmax = float(rmax)
  tmin = 0.0
  tmax = 2*pi  
  zmin = float(zmin)
  zmax = float(zmax)
      
  ir = arange(nr) 
  it = arange(nt)
  iz = arange(nz)
 
  r = (ir+offr)*(rmax-rmin)/nr + rmin  
  t = (it+offt)*(tmax-tmin)/nt + tmin 
  z = (iz+offz)*(zmax-zmin)/nz + zmin 

  return r,t,z


def get_Points_On_Cylindrical_3d_Grid(nr,nt,nz,rmax,zmin,zmax,offr=0,offt=0,offz=0):
  '''
  Return an array of points corresponding to the nodes of
  a 2d cylindrical grid
  
  To get a nt X nr array from the returned vector (pos), do
  
  x = pos[:,0]
  y = pos[:,0]
  z = pos[:,0]
  
  x.shape = (nr,nt,nz)
  y.shape = (nr,nt,nz)
  z.shape = (nr,nt,nz)
  
  # to get r and theta
  r = sqrt(x**2+y**2+z**2)
  t = arctan2(y,x)*180/pi
    
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  tmin = 0.0
  tmax = 2*pi
  zmin = float(zmin)
  zmax = float(zmax)
  nr   = int(nr)
  nt   = int(nt)
  nz   = int(nz)
  
  ir,it,iz = indices((nr,nt,nz))

  r = (ir+offr)*(rmax-rmin)/nr + rmin
  t = (it+offt)*(tmax-tmin)/nt + tmin
  z = (iz+offz)*(zmax-zmin)/nz + zmin

  r = ravel(r)
  t = ravel(t)
  z = ravel(z)

  x = r*cos(t)
  y = r*sin(t)
  z = z

  return transpose(array([x,y,z])).astype(Float32)



def get_MassMap_On_Cylindrical_3d_Grid(nb,nr,nt,nz,rmax,zmin,zmax):
  '''
  Return an array of points containing mass of particles
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  tmin = 0.0
  tmax = 2*pi
  zmin = float(zmin)
  zmax = float(zmax)
  
  r = nb.rxy()
  t = nb.phi_xy() + pi
  z = nb.pos[:,2]
    
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  t   = (t-tmin)/(tmax-tmin)
  z   = (t-zmin)/(zmax-zmin)
  
  # compute values
  pos = transpose(array([r,t,z])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= nb.mass.astype(Float32) 
  shape = (nr,nt,nz)
  
  # make the map  
  mat = mapping.mkmap3d(pos,mass,val,shape)
  
  return mat

def get_NumberMap_On_Cylindrical_3d_Grid(nb,nr,nt,nz,rmax,zmin,zmax):
  '''
  Return an array of points containing mass of particles
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  tmin = 0.0
  tmax = 2*pi
  zmin = float(zmin)
  zmax = float(zmax)
  
  r = nb.rxy()
  t = nb.phi_xy() + pi
  z = nb.pos[:,2]
    
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  t   = (t-tmin)/(tmax-tmin)
  z   = (t-zmin)/(zmax-zmin)
  
  # compute values
  pos = transpose(array([r,t,z])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= ones(pos.shape).astype(Float32)
  shape = (nr,nt,nz)
  
  # make the map  
  mat = mapping.mkmap3d(pos,mass,val,shape)
  
  return mat

def get_PotentialMap_On_Cylindrical_3d_Grid(nb,nr,nt,nz,rmax,zmin,zmax,eps,Tree=None):
  '''
  Return an array of points containing potential   
  '''

  pos = get_Points_On_Cylindrical_3d_Grid(nr,nt,nz,rmax,zmin,zmax)
  
  Tree = nb.getTree()
    
  pot = Tree.Potential(pos,eps)
  
  # transform back into array  
  pot.shape = (nr,nt,nz)
  
  return pot  


def get_VolumeMap_On_Cylindrical_3d_Grid(nb,nr,nt,nz,rmax,zmin,zmax):
  '''
  Return an array of points containing corresponding physical 
  volumes of each cell (usefull to compute density)
  '''
  
  rmax = float(rmax)
  zmin = float(zmin)
  zmax = float(zmax)
    
  # compute volume (surface)
  #dr = rmax/nr
  #rs = arange(nr+1)*dr
  #v = (pi*rs[1:]**2 - pi*rs[:-1]**2 )/nt
     
  def volume(ir,it,iz):
    dr = (rmax/nr)
    dz = (zmax-zmin)/nz
    r  = ir*dr
    return pi*((r+dr)**2 - (r)**2 )/nt * dz
 
  # make the map  
  mat = fromfunction(volume,(nr,nt,nz))
  mat = mat.astype(Float32)
    
  return mat


def get_DensityMap_On_Cylindrical_3d_Grid(nb,nr,nt,nz,rmax,zmin,zmax):
  '''
  Return an array of points containing density in each cell
  '''
    
  m = get_MassMap_On_Cylindrical_3d_Grid(nb,nr,nt,nz,rmax,zmin,zmax)
  v = get_VolumeMap_On_Cylindrical_3d_Grid(nb,nr,nt,nz,rmax,zmin,zmax)
  
  return m/v


  
#######################################
# spherical grid
#######################################


##############
# 1 dimension
##############

def get_r_Of_Spherical_1d_Grid(nr,rmax,offr=0):

  rmin = 0.0
  rmax = float(rmax)
      
  ir = arange(nr) 
 
  r = (ir+offr)*(rmax-rmin)/nr + rmin
  
  return r


def get_Points_On_Spherical_1d_Grid(nr,rmax,offr=0):
  '''
  Return an array of points corresponding to the nodes of
  a 1d spherical grid
  
  To get a nt X nr array from the returned vector (pos), do
  
  x = pos[:,0]
  y = pos[:,0]
  z = pos[:,0]
  
  x.shape = (nr,np,nt)
  y.shape = (nr,np,nt)
  z.shape = (nr,np,nt)
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  nr   = int(nr)
  
  ir = indices((nr,))
  
  r = (ir+offr)*(rmax-rmin)/nr + rmin

  r = ravel(r)

  x = r
  y = zeros(len(x))
  z = zeros(len(x))
  
  return transpose(array([x,y,z])).astype(Float32)


def get_MassMap_On_Spherical_1d_Grid(nb,nr,rmax):
  '''
  Return an array of points containing mass of particles
  '''

  rmin = 0.0
  rmax = float(rmax)

  r = nb.rxyz()			# 0 -> rmax
  
  
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  
  # compute values
  pos = transpose(array(r)).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= nb.mass.astype(Float32) 
  shape = (nr,)
  
  # make the map  
  mat = mapping.mkmap1d(pos,mass,val,shape)
  
  return mat

def get_NumberMap_On_Spherical_1d_Grid(nb,nr,rmax):
  '''
  Return an array of points containing mass of particles
  '''

  rmin = 0.0
  rmax = float(rmax)

  r = nb.rxyz()			# 0 -> rmax
  
  
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  
  # compute values
  pos = transpose(array(r)).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= ones(pos.shape).astype(Float32) 
  shape = (nr,)
  
  # make the map  
  mat = mapping.mkmap1d(pos,mass,val,shape)
  
  return mat


def get_PotentialMap_On_Spherical_1d_Grid(nb,nr,rmax,eps,Tree=None):
  '''
  Return an array of points containing potential   
  '''

  pos = get_Points_On_Spherical_1d_Grid(nr,rmax)
  
  Tree = nb.getTree()
    
  pot = Tree.Potential(pos,eps)
  
  # transform back into array  
  pot.shape = (nr,)

  
  return pot

def get_SufaceMap_On_Spherical_1d_Grid(nb,nr,rmax):
  '''
  Return an array of points containing surface (volume)
  of each cell.
  '''  
  rmin = 0.0
  rmax = float(rmax)

  # compute volume (surface)
     
  def volume(ir):
    dr = (rmax/nr) + (ir-ir)
    return dr
 
  # make the map  
  mat = fromfunction(volume,(nr,))
  mat = mat.astype(Float32)
    
  return mat


def get_VolumeMap_On_Spherical_1d_Grid(nb,nr,rmax):
  '''
  Return an array of points containing corresponding physical 
  volumes of each cell (usefull to compute density)
  '''  
  rmin = 0.0
  rmax = float(rmax)

    
  # compute volume (surface)
     
  def volume(ir):
    dr = (rmax/nr)
    r  = ir*dr
    return 4.0/3.0*pi*( (r+dr)**3 - (r)**3 )
 
  # make the map  
  mat = fromfunction(volume,(nr,))
  mat = mat.astype(Float32)
    
  return mat


def get_DensityMap_On_Spherical_1d_Grid(nb,nr,rmax):
  '''
  Return an array of points containing density in each cell
  '''
    
  m = get_MassMap_On_Spherical_1d_Grid(nb,nr,rmax)
  v = get_VolumeMap_On_Spherical_1d_Grid(nb,nr,rmax)
  
  return m/v


def get_LinearDensityMap_On_Spherical_1d_Grid(nb,nr,rmax):
  '''
  Return an array of points containing the linear density in each cell
  '''
    
  m = get_MassMap_On_Spherical_1d_Grid(nb,nr,rmax)
  s = get_SurfaceMap_On_Spherical_1d_Grid(nb,nr,rmax)
  
  return m/s


def get_AccumulatedMassMap_On_Spherical_1d_Grid(nb,nr,rmax):
  '''
  Return an array of points containing M(r) in each cell
  '''
    
  m   = get_MassMap_On_Spherical_1d_Grid(nb,nr,rmax)
  mr  = get_Accumulation_Along_Axis(m,axis=0)
  
  return mr



def get_Interpolation_On_Spherical_1d_Grid(pos,mat,nr,rmax,offr=0):
  '''
  Interpolates continuous value of pos, using matrix mat
  '''
    
  rmin = 0.0
  rmax = float(rmax)
  nr   = int(nr)  
    
  r  = sqrt(pos[:,0]**2+pos[:,1]**2+pos[:,2]**2)
  ir = (r-rmin)/(rmax-rmin)* nr - offr
    
  return myNumeric.Interpolate_From_1d_Array(ir.astype(Float32),mat.astype(Float32))


##############
# 3 dimensions
##############



def get_rpt_Of_Spherical_3d_Grid(nr,np,nt,rmax,offr=0,offp=0,offt=0):

  rmin = 0.0
  rmax = float(rmax)
  pmin = 0.0
  pmax = 2*pi
  tmin = -pi/2
  tmax = pi/2
      
  ir = arange(nr) 
  ip = arange(np)
  it = arange(nt)
 
  r = (ir+offr)*(rmax-rmin)/nr + rmin
  p = (ip+offp)*(pmax-pmin)/np + pmin   
  t = (it+offt)*(tmax-tmin)/nt + tmin 
  
  return r,p,t
  
def get_Points_On_Spherical_3d_Grid(nr,np,nt,rmax,offr=0,offp=0,offt=0):
  '''
  Return an array of points corresponding to the nodes of
  a 3d spherical grid
  
  To get a nt X nr array from the returned vector (pos), do
  
  x = pos[:,0]
  y = pos[:,0]
  z = pos[:,0]
  
  x.shape = (nr,np,nt)
  y.shape = (nr,np,nt)
  z.shape = (nr,np,nt)
  '''
  
  rmin = 0.0
  rmax = float(rmax)
  pmin = 0.0
  pmax = 2*pi
  tmin = -pi/2
  tmax = pi/2
  nr   = int(nr)
  np   = int(np)
  nt   = int(nt)
  
  ir,ip,it = indices((nr,np,nt))
  
  r = (ir+offr)*(rmax-rmin)/nr + rmin
  p = (ip+offp)*(pmax-pmin)/np + pmin
  t = (it+offt)*(tmax-tmin)/nt + tmin

  r = ravel(r)
  p = ravel(p)
  t = ravel(t)

  x = r*cos(p)*cos(t)
  y = r*sin(p)*cos(t)
  z = r       *sin(t)
  
  return transpose(array([x,y,z])).astype(Float32)


def get_MassMap_On_Spherical_3d_Grid(nb,nr,np,nt,rmax):
  '''
  Return an array of points containing mass of particles
  '''

  rmin = 0.0
  rmax = float(rmax)
  pmin = 0.0
  pmax = 2*pi
  tmin = -pi/2
  tmax = pi/2


  r = nb.rxyz()			# 0 -> rmax
  p = nb.phi_xyz() + pi		# 0 -> pi
  t = nb.theta_xyz() 		# -pi/2 -> pi/2
  
  
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  p   = (p-pmin)/(pmax-pmin)
  t   = (t-tmin)/(tmax-tmin)
  
  # compute values
  pos = transpose(array([r,p,t])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= nb.mass.astype(Float32) 
  shape = (nr,np,nt)
  
  # make the map  
  mat = mapping.mkmap3d(pos,mass,val,shape)
  
  return mat

def get_NumberMap_On_Spherical_3d_Grid(nb,nr,np,nt,rmax):
  '''
  Return an array of points containing mass of particles
  '''

  rmin = 0.0
  rmax = float(rmax)
  pmin = 0.0
  pmax = 2*pi
  tmin = -pi/2
  tmax = pi/2


  r = nb.rxyz()			# 0 -> rmax
  p = nb.phi_xyz() + pi		# 0 -> pi
  t = nb.theta_xyz() 		# -pi/2 -> pi/2
  
  
  # scale between 0 and 1
  r   = (r-rmin)/(rmax-rmin)
  p   = (p-pmin)/(pmax-pmin)
  t   = (t-tmin)/(tmax-tmin)
  
  # compute values
  pos = transpose(array([r,p,t])).astype(Float32)
  val = ones(pos.shape).astype(Float32) 
  mass= ones(pos.shape).astype(Float32)
  shape = (nr,np,nt)
  
  # make the map  
  mat = mapping.mkmap3d(pos,mass,val,shape)
  
  return mat


def get_PotentialMap_On_Spherical_3d_Grid(nb,nr,np,nt,rmax,eps,Tree=None):
  '''
  Return an array of points containing potential   
  '''

  pos = get_Points_On_Spherical_3d_Grid(nr,np,nt,rmax)
  
  Tree = nb.getTree()
    
  pot = Tree.Potential(pos,eps)
  
  # transform back into array  
  pot.shape = (nr,np,nt)

  
  return pot


def get_VolumeMap_On_Spherical_3d_Grid(nb,nr,np,nt,rmax):
  '''
  Return an array of points containing corresponding physical 
  volumes of each cell (usefull to compute density)
  '''
  
  print "get_VolumeMap_On_Spherical_3d_Grid is probably wrong"
  print "we should add a cos(t) or somthing similar"
  sys.exit()
  
  rmin = 0.0
  rmax = float(rmax)
  pmin = 0.0
  pmax = 2*pi
  tmin = -pi/2
  tmax = pi/2

    
  # compute volume (surface)
     
  def volume(ir,ip,it):
    dr = (rmax/nr)
    r  = ir*dr
    return 4.0/3.0*pi*( (r+dr)**3 - (r)**3 ) / np / nt
 
  # make the map  
  mat = fromfunction(volume,(nr,np,nt))
  mat = mat.astype(Float32)
    
  return mat


def get_DensityMap_On_Spherical_3d_Grid(nb,nr,np,nt,rmax):
  '''
  Return an array of points containing density in each cell
  '''
    
  m = get_MassMap_On_Spherical_3d_Grid(nb,nr,np,nt,rmax)
  v = get_VolumeMap_On_Spherical_3d_Grid(nb,nr,np,nt,rmax)
  
  return m/v
